﻿unit U_FormCopy;

{$IFDEF FPC}
{$mode Delphi}
{$R *.lfm}
{$ELSE}
{$R *.dfm}
{$ENDIF}


interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs,
{$IFDEF FPC}
  LResources, Process, SdfData, db, EditBtn,
{$ELSE}
  Mask, rxToolEdit, JvExControls,
{$ENDIF}
  U_ExtFileCopy, ExtCtrls, FileCtrl, StdCtrls, U_OnFormInfoIni,
   ComCtrls, Menus, JvXPButtons,
  UTF8Process,
  U_FileExecute, U_FormMainIni ;

type

  { TF_ExeFiles }

  TF_ExeFiles = class(TF_FormMainIni)
    bt_open: TJvXPButton;
    ch_subdirs: TCheckbox;
    ch_structure: TCheckBox;
    EndLine: TEdit;
    bt_Extract: TJvXPButton;
    BeginLine: TEdit;
    EndExtract2: TEdit;
    FileExecute: TFileExecute;
    ExeFile: TFileNameEdit;
    FilesSeek: TExtFileCopy;
    FDestination: TDirectoryEdit;
    Label1: TLabel;
    OnFormInfoIni: TOnFormInfoIni;
    Panel5: TPanel;
    Panel9: TPanel;
    ProcessUTF8: TProcessUTF8;
    ProgressBar: TProgressBar;
    Result: TMemo;
    Panel7: TPanel;
    Panel1: TPanel;
    Panel3: TPanel;
    DirectorySource: TDirectoryEdit;
    FileListSource: TFileListBox;
    Panel6: TPanel;
    Mask: TEdit;
    ResultErrors: TMemo;
    Panel2: TPanel;
    MainMenu: TMainMenu;
    Language: TMenuItem;
    Aide: TMenuItem;
    Faireundon: TMenuItem;
    APropos: TMenuItem;
    Splitter1: TSplitter;
    Splitter2: TSplitter;
    Panel8: TPanel;
    pa_DestImages: TPanel;
    Splitter3: TSplitter;
    Panel4: TPanel;
    procedure bt_ExtractClick(Sender: TObject);
    procedure bt_openClick(Sender: TObject);
    procedure DirectorySourceChange(Sender: TObject);
    procedure FileExecuteCopy(Sender: Tobject; const ASource, AExtensionSource,
      ADestination: String; var as_LineWithoutExe: String;var AContinue:Boolean);
    procedure FilesSeekChange(Sender: Tobject; const NewDirectory, DestinationDirectory : String);
    procedure FilesSeekFailure(Sender: Tobject; const ErrorCode: Integer;
      var ErrorMessage: AnsiString; var ContinueCopy: Boolean);
    procedure FilesSeekProgress(Sender: Tobject; BytesCopied,
      BytesTotal: cardinal);
    procedure FormDestroy(Sender: TObject);
    procedure MaskChange(Sender: TObject);
    procedure FilesSeekSuccess(Sender: TObject; const ASource,
      ADestination: string; const Errors: Integer);
    procedure LanguageClick(Sender: TObject);
    procedure FaireundonClick(Sender: TObject);
    procedure AProposClick(Sender: TObject);
    procedure TraduceFileSuccess(Sender: TObject; const ASource,
      ADestination: string; const Errors: Integer);
  private
{$IFDEF FPC}
    Donate : TProcess;
{$ENDIF}
    FTraduitVers : String ;
    FCopieVers : String ;
    { private declarations }
  public
    constructor Create ( Aowner : TCOmponent ); override;
    procedure ChargeLangue(const ALangue: String);
    { public declarations }
  end;

var
  F_ExeFiles: TF_ExeFiles;
  FLangue : String ;

implementation

uses fonctions_ini, IniFiles,
     fonctions_system,
     fonctions_string,
     fonctions_file,
     {$IFDEF FPC}
     LazFileUtils,
     LCLType,
     {$ELSE}
     Windows, ShellApi,
     {$ENDIF}
     FileUtil,
     U_ABOUTBOX;

const
  CST_SITE_DONATE = 'http://matthieu.giroux.free.fr/html/donate.html';

{ TF_ExeFiles }

procedure TF_ExeFiles.AProposClick(Sender: TObject);
begin
  F_AboutBox.ShowModal;
end;

procedure TF_ExeFiles.bt_ExtractClick(Sender: TObject);
begin
  if not DirectoryExistsUTF8(ExtractFileDir(FDestination.Text))
  and not DirectoryExistsUTF8(FileListSource.Directory)
   Then
    Exit;

  if not DirectoryExistsUTF8 ( FDestination.Text ) Then
    ForceDirectoriesUTF8(FDestination.Text);

  with FileExecute do
   Begin
    Exe:=ExeFile.FileName;
   end;
  with FilesSeek do
   Begin
    Mask:=Self.Mask.Text;
    if ch_subdirs.Checked
     then FilesOptions:=FilesOptions+[cpCopyAll]
     Else FilesOptions:=FilesOptions-[cpCopyAll];
    if ch_structure.Checked
     then FilesOptions:=FilesOptions-[cpNoStructure]
     Else FilesOptions:=FilesOptions+[cpNoStructure];
    Source := FileListSource.Directory ;
    Destination:=FDestination.Directory;
    CopySourceToDestination;
   end;
end;

procedure TF_ExeFiles.bt_openClick(Sender: TObject);
begin
  p_OpenFileOrDirectory(FDestination.Directory);
end;


procedure TF_ExeFiles.DirectorySourceChange(Sender: TObject);
begin
    FileListSource.Directory := DirectorySource.Text;

end;


procedure TF_ExeFiles.FileExecuteCopy(Sender: Tobject; const ASource, AExtensionSource,
  ADestination: String; var as_LineWithoutExe: String;var AContinue:Boolean);
begin
  if  ( AExtensionSource <> 'pas' )
  and ( AExtensionSource <> 'pp' )
  and ( AExtensionSource <> 'inc' )
   Then AContinue:=False
   Else as_LineWithoutExe:=ASource+' '+ADestination;
end;

procedure TF_ExeFiles.FaireundonClick(Sender: TObject);
begin
{$IFDEF FPC}
  Donate.Execute;
{$ELSE}
  shellexecute ( Handle, 'Open', CST_SITE_DONATE, '', '.', 0 );
{$ENDIF}
end;

procedure TF_ExeFiles.FilesSeekChange(Sender: Tobject; const NewDirectory, DestinationDirectory : String);
begin
  if ( NewDirectory <> FileListSource.Directory )
    then
      FileListSource.Directory := NewDirectory;
end;

procedure TF_ExeFiles.FilesSeekFailure(Sender: Tobject; const ErrorCode: Integer;
  var ErrorMessage: AnsiString; var ContinueCopy: Boolean);
begin
  ResultErrors.Lines.BeginUpdate ;
  ResultErrors.Lines.Add ( ErrorMessage );
  ResultErrors.Lines.EndUpdate ;
  case MessageDlg({$IFDEF FPC}'Erreur '+IntToStr(ErrorCode),{$ENDIF}ErrorMessage, mtError, [mbAbort, mbRetry,mbIgnore], 0 ) of
     mrAbort : abort;
     mrIgnore: ContinueCopy := False;
     mrRetry : ContinueCopy := True;
  end;
end;

procedure TF_ExeFiles.FilesSeekProgress(Sender: Tobject; BytesCopied,
  BytesTotal: cardinal);
begin
  ProgressBar.Max := BytesTotal;
  ProgressBar.Position := BytesCopied;
end;

procedure TF_ExeFiles.FilesSeekSuccess(Sender: TObject; const ASource,
  ADestination: string; const Errors: Integer);
begin
  Result.Lines.BeginUpdate ;
  Result.Lines.Add ( ASource + ' ' + FCopieVers + ' ' + ADestination  );
  if ( Result.Lines.Count > 50 )
   Then
    Result.Lines.Delete ( 0 );
  Result.Lines.EndUpdate ;

end;


procedure TF_ExeFiles.FormDestroy(Sender: TObject);
begin
  if Assigned ( FIniFile ) then
      FIniFile.WriteString  ( 'Parametres', 'Language'  , FLangue );
end;


procedure TF_ExeFiles.ChargeLangue(const ALangue : String );
var
   FFichierIni ,
   Ftempo : String ;
   FIniLangue : TIniFile ;
   i: Integer ;
begin
  FLangue := ALangue ;
  FFichierIni := ExtractFilePath(Application.ExeName) + 'Languages' +DirectorySeparator + FLangue + '.lng' ;

  if ( FileExistsUTF8 ( FFichierIni )) then
    Begin
      FIniLangue := TIniFile.Create ( FFichierIni );
      if assigned ( FIniLangue ) then
        Begin
          bt_Extract     .Caption := FIniLangue.ReadString ( 'Buttons', 'Extract', 'Copy' );
          ch_subdirs     .Caption := FIniLangue.ReadString ( 'Checks', 'Subdirectories', 'Subdirectories' );
          Language    .Caption := FIniLangue.ReadString ( 'Menu', 'Language', 'Language' );
          Faireundon  .Caption := FIniLangue.ReadString ( 'Menu', 'Donate', 'Donate' );
          Aide        .Caption := FIniLangue.ReadString ( 'Menu', 'Help', 'Help' );
          APropos     .Caption := FIniLangue.ReadString ( 'Menu', 'About', 'About' );
          F_AboutBox.Comments.Caption := FIniLangue.ReadString ( 'F_About', 'Comments', 'Copy of Files with Images Traducing' );
          F_AboutBox.OKButton.Caption := FIniLangue.ReadString ( 'F_About', 'OK', 'OK' );
          FTraduitVers := FIniLangue.ReadString ( 'Copy', 'Traduced to', 'traduced to' );
          FCopieVers   := FIniLangue.ReadString ( 'Copy', 'Copied to', 'copied to' );
         // GS_COPYFILES_CONFIRM := FIniLangue.ReadString ( 'Copy', 'COPYFILES_CONFIRM', 'Confirm Box' );
         // GS_COPYFILES_CONFIRM_FILE_DELETE := FIniLangue.ReadString ( 'Copy', 'COPYFILES_CONFIRM_FILE_DELETE', 'Do you really want to erase the file' ) + ' ' ;

          FIniLangue.Free ;
        End;
    End ;
End ;
procedure TF_ExeFiles.LanguageClick(Sender: TObject);
var Ftempo  ,
    FLangue : String ;
    i : Integer ;
begin
  If ( Sender is TMenuItem ) Then
    Begin
      Ftempo := ( Sender as TMenuItem ).Caption ;
      for I := 1 to length ( Ftempo ) do
        if (Ftempo [i] <> '&' ) then
          FLangue := FLangue + Ftempo [I];
      ChargeLangue( FLangue );
    End;
end;

procedure TF_ExeFiles.MaskChange(Sender: TObject);
begin
  FileListSource.Mask := Mask.text ;;
end;

procedure TF_ExeFiles.TraduceFileSuccess(Sender: TObject; const ASource,
  ADestination: string; const Errors: Integer);
begin
  Result.Lines.BeginUpdate ;
  Result.Lines.Add ( ASource + ' '+ FTraduitVers + ' ' + ADestination  );
  if ( Result.Lines.Count > 50 )
   Then
    Result.Lines.Delete ( 0 );
  Result.Lines.EndUpdate ;
end;

constructor TF_ExeFiles.Create(Aowner: TComponent);
begin
  AutoIni:=True;
  inherited;
{$IFDEF FPC}
  if not ( csDesigning in ComponentState ) Then
    Begin
      Donate := TProcess.Create ( Self );
      Donate.ApplicationName := 'open' ;
      Donate.CommandLine := 'open ' + CST_SITE_DONATE ;
    end;
{$ENDIF}
end;

end.

