unit u_extradios;

{$IFDEF FPC}
{$mode Delphi}{$H+}
{$ENDIF}

interface

uses
  Classes, SysUtils,
{$IFDEF FPC}
  LResources,
{$ELSE}
{$ENDIF}
   Forms, Controls, Graphics,
  Dialogs, ExtCtrls,DbCtrls;

const
    CST_RULE_PROCEDURE_ITEMCLICK = 'p_ItemClick';
{$IFDEF VERSIONS}
    gVer_extradios : T_Version = ( Component : 'Radio Components with Rules' ;
                                      FileUnit : 'U_ExtRadios' ;
                                      Owner : 'Matthieu Giroux' ;
                                      Comment : 'Radio Components with Rules, Values.' ;
                                      BugsStory : '0.9.0.0 : Not fully tested.' + #13#10 ;
                                       UnitType : 3 ;
                                       Major : 0 ; Minor : 9 ; Release : 0; Build : 0 );

{$ENDIF}

type
  TExtRadioGroup = class;
  TFWRules       = class;

  TSetEdit = (seNone,seEnable,seDisable,seEditable,seReadonly,seNeeded,seOption,
              seClearNotChecked,seClearChecked,seSetValueChecked,seSetValueNotChecked,
              seItemCheckNotChecked,seItemCheckChecked,seItemUncheckNotChecked,seItemUncheckChecked,
              seItemInvertNotChecked,seItemInvertChecked);
  IRuleComponent = interface
     ['{BD26CD63-FA01-447B-8311-E3E41136253D}'] // GUID nécessaire pour l'opération de cast
     function ffwr_CreateRules: TFWRules;  // Need published Rule Property
    End;

  { TFWRule }

  TFWRule = class(TCollectionItem)
  private
    FOnChange : TNotifyEvent ;
    FEdit : TControl ;
    FSetValue,
    FItemIndex: Integer;
    FSetEdit : TSetEdit;
    FFieldName : String;
    procedure p_SetEdit( const a_Value: TControl);
    procedure p_SetRule( const a_Value: TSetEdit);
    procedure p_SetValue( const a_Value: Integer);
    procedure p_SetItemIndex( const a_Value: Integer);
  protected
    constructor Create(Collection: TCollection);override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); virtual;
    procedure Changed; virtual;
    procedure p_SetRuleToControl (const AItemIndex : Integer);
  public
  published
   property ItemIndex : Integer  read FItemIndex write p_SetItemIndex;
   property SetValue : Integer  read FSetValue write p_SetValue;
   property Edit : TControl  read FEdit write p_SetEdit;
   property FieldName : String  read FFieldName write FFieldName;
   property OnChange : TNotifyEvent  read FOnChange write FOnChange;
   property Rule : TSetEdit  read FSetEdit write p_SetRule default seNone;
  end;
  TFWRuleClass = class of TFWRule;

  { TFWRules }

  TFWRules = class(TCollection)
  private
    FParent : TComponent;
    FRules : TFWRules;
    function GetRuleColumn( Index: Integer): TFWRule;
    procedure SetRuleColumn( Index: Integer; Value: TFWRule);
  protected
    function GetOwner: TPersistent; override;
  public
    function Add: TFWRule;
    constructor Create(const Source: TComponent; const ColumnClass: TFWRuleClass); virtual;
    property Parent: TComponent read FParent;
  {$IFDEF FPC}
  published
  {$ENDIF}
    property Items[Index: Integer]: TFWRule read GetRuleColumn write SetRuleColumn; default;
  End;

  { TExtRadioGroup }

  TExtRadioGroup = class(TRadioGroup,IRuleComponent)
  private
    FValues : TStrings;
    FRules : TFWRules;
    p_setPanels: TFWRules;
    procedure p_setRules(const AValue: TFWRules);
    procedure p_setValues(const AValues: TStrings);
    { Private declarations }
  protected
    { Protected declarations }
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function ffwr_CreateRules: TFWRules;
  public
    { Public declarations }
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;
    procedure CheckItemIndexChanged; override;
  published
    { Published declarations }
   property Rules  : TFWRules read FRules  write p_setRules  ;
   property Values  : TStrings read FValues  write p_setValues  ;

  end;

  { TExtCheckGroup }

  TExtCheckGroup = class(TCheckGroup,IRuleComponent)
  private
    FRules : TFWRules;
    p_setPanels: TFWRules;
    FOldItemClick : TCheckGroupClicked;
    procedure p_setRules(const AValue: TFWRules);
    { Private declarations }
  protected
    { Protected declarations }
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function ffwr_CreateRules: TFWRules;
  public
    { Public declarations }
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;
    procedure p_ItemClick (Sender: TObject; Index: integer); virtual;
    procedure Loaded; override;
  published
    { Published declarations }
    property Rules  : TFWRules read FRules  write p_setRules  ;

  end;

  { TExtDBRadioGroup }

  TExtDBRadioGroup = class(TDBRadioGroup,IRuleComponent)
  private
    FRules : TFWRules;
    p_setPanels: TFWRules;
    procedure p_setRules(const AValue: TFWRules);
    { Private declarations }
  protected
    { Protected declarations }
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function ffwr_CreateRules: TFWRules;
  public
    { Public declarations }
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;
    procedure CheckItemIndexChanged; override;
  published
    { Published declarations }
    property Rules  : TFWRules read FRules  write p_setRules  ;

  end;

const TItemCheckOnlyEdit : set of TSetEdit= [seItemCheckNotChecked..seItemInvertChecked];
      TItemUncheckOnlyEdit : set of TSetEdit= [seItemUncheckNotChecked..seItemUncheckChecked];

implementation

uses fonctions_proprietes,typinfo,DB;

{ TFWRules }

function TFWRules.GetRuleColumn(Index: Integer): TFWRule;
begin
  if (Index > -1)
  and (Index<Count) Then
   Result:=Inherited Items[Index] as TFWRule;
end;

procedure TFWRules.SetRuleColumn(Index: Integer; Value: TFWRule);
begin
  if (Index > -1)
  and (Index<Count) Then
    Inherited Items[Index]:=Value;
end;

function TFWRules.GetOwner: TPersistent;
begin
  Result:=FParent;
end;

function TFWRules.Add: TFWRule;
begin
  Result:= Inherited Add as TFWRule;
end;

constructor TFWRules.Create(const Source: TComponent;
  const ColumnClass: TFWRuleClass);
begin
  Inherited Create(ColumnClass);
  FParent:=Source;
end;

{ TFWRule }

procedure TFWRule.p_SetEdit(const a_Value: TControl);
begin
  if  ( a_Value <> FEdit )
  and ( a_Value <> Collection.Owner ) Then
   FEdit:=a_Value;
end;

procedure TFWRule.p_SetRule(const a_Value: TSetEdit);
begin
  FSetEdit:=a_Value;
end;

procedure TFWRule.p_SetValue(const a_Value: Integer);
begin
  FSetValue:=a_Value;
end;

procedure TFWRule.p_SetItemIndex(const a_Value: Integer);
begin
  FItemIndex:=a_Value;
end;

constructor TFWRule.Create(Collection: TCollection);
begin
  inherited;
  FSetEdit:= seNone;
  FEdit:=nil;
end;

procedure TFWRule.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if    Assigned   ( Edit  )
  and ( AComponent = Edit )
   then
    Edit := nil;
end;

procedure TFWRule.Changed;
begin
  if Assigned(FOnChange)
   Then
    FOnChange(Self);
end;

procedure TFWRule.p_SetRuleToControl (const AItemIndex : Integer);
var LEvent : TNotifyEvent;
    LItemEvent : TCheckGroupClicked;
    //Clear External Control
  procedure p_Clear;
  Begin
    if assigned ( GetPropInfo ( FEdit, CST_PROPERTY_TEXT ))
      then SetPropValue    ( FEdit, CST_PROPERTY_TEXT, '')
     else if assigned ( GetPropInfo ( FEdit, CST_PROPERTY_VALUE ))
      then SetPropValue    ( FEdit, CST_PROPERTY_VALUE, -1);
  End;
  // Set Internal Datasource
  procedure p_SetDatasetWithItems;
  var lstr_Items : TStrings;
      ldat_Datasource : TDatasource ;
  Begin
    with Collection do
     try
       lstr_Items      := TStrings(fobj_getComponentObjectProperty(Owner,CST_PROPERTY_ITEMS));
       ldat_Datasource := TDataSource(fobj_getComponentObjectProperty(Owner,CST_DBPROPERTY_DATASOURCE));
       ldat_Datasource.DataSet.FieldByName(fs_getComponentProperty(Owner,CST_DBPROPERTY_DATAFIELD)).AsString:=lstr_Items[SetValue];
     finally
     end;
  end;
  // Change from rule
  procedure p_ItemSet;
  var li_i : Integer;
  Begin
    with Collection,Owner do
     if Owner is TCustomCheckGroup Then
      with Owner as TCustomCheckGroup do
       Begin
         if FSetEdit in TItemCheckOnlyEdit Then
           Begin
            if SetValue < 0 Then
             Begin   // Uncheck everything
               if FSetEdit in TItemUncheckOnlyEdit Then
                 for li_i := 0 to Rows - 1 do
                  Checked[li_i] := False
             end
            Else // Check or uncheck
             if SetValue < Rows Then
              case FSetEdit of
               seItemCheckChecked,seItemCheckNotChecked : Checked[SetValue] := True;
               seItemInvertChecked,seItemInvertNotChecked : Checked[SetValue] := not Checked[SetValue];
               seItemUncheckChecked,seItemUncheckNotChecked : Checked[SetValue] := False;
              End;
          End
         Else  // Enable or disable
          if  ( SetValue > 0 )
          and ( SetValue < Rows )
           Then
             case FSetEdit of
              seEnable   : CheckEnabled[SetValue] := True;
              seDisable  : CheckEnabled[SetValue] := False;
             End;
       End
     Else  // Set radio
      if  ( FSetEdit in TItemCheckOnlyEdit )
      and ( SetValue > -2 ) Then
       if IsPublishedProp(Owner,CST_DBPROPERTY_DATASOURCE)
        Then p_SetDatasetWithItems
        Else p_SetComponentProperty(Owner,CST_PROPERTY_ITEMINDEX,FSetValue);
  End;
begin
  if ( FItemIndex = AItemIndex ) Then   // Is it the good rule ?
   Begin
    if Assigned(FEdit)
     Then // External Control rules
      Begin
        case FSetEdit of
         seEditable:p_SetComponentBoolProperty(FEdit,CST_PROPERTY_READONLY,False);
         seReadonly:p_SetComponentBoolProperty(FEdit,CST_PROPERTY_READONLY,True);
         seEnable:FEdit.Enabled:=True;
         seDisable:FEdit.Enabled:=False;
         seClearChecked:p_Clear;
         seClearNotChecked: Begin
                             LEvent:=TNotifyEvent(fmet_getComponentMethodProperty(FEdit,CST_PROPERTY_ONCHANGE));
                             p_SetComponentMethodNameProperty(FEdit,CST_PROPERTY_ONCHANGE,nil,'');
                             p_Clear;
                             p_SetComponentMethodProperty(FEdit,CST_PROPERTY_ONCHANGE,TMethod(LEvent));
                            End;
         seSetValueChecked:p_SetComponentProperty(FEdit,CST_PROPERTY_VALUE,FSetValue);
         seSetValueNotChecked:Begin
                               LEvent:=TNotifyEvent(fmet_getComponentMethodProperty(FEdit,CST_PROPERTY_ONCLICK));
                               p_SetComponentMethodNameProperty(FEdit,CST_PROPERTY_ONCLICK,nil,'');
                               p_SetComponentProperty(FEdit,CST_PROPERTY_VALUE,FSetValue);
                               p_SetComponentMethodProperty(FEdit,CST_PROPERTY_ONCLICK,TMethod(LEvent));
                              End;
        end;
      end
   Else // Internal Rules
    if FSetValue <> AItemIndex Then
     Begin
       case FSetEdit of
        seItemCheckNotChecked,seItemUncheckNotChecked,seItemInvertNotChecked:
          with Collection do
           Begin
            LItemEvent:=TCheckGroupClicked(fmet_getComponentMethodProperty(Owner,CST_RULE_PROCEDURE_ITEMCLICK));
            p_SetComponentMethodNameProperty(Owner,CST_RULE_PROCEDURE_ITEMCLICK,nil,'');
            p_ItemSet;
            p_SetComponentMethodProperty(Owner,CST_RULE_PROCEDURE_ITEMCLICK,TMethod(LItemEvent));
           End;
        seItemCheckChecked,seItemUncheckChecked,seDisable,seEnable,seItemInvertChecked:
          p_ItemSet;
       end;
     end;
    if FSetEdit > seNone
     Then Changed; // Changed
   End;
end;

{------------------------------------------------------------------------------
  Method: TExtRadioGroup.p_setRules
  Params:  avalue - Rules' collection
  Returns: Nothing

  Assign items from a stringlist.
 ------------------------------------------------------------------------------}
procedure TExtRadioGroup.p_setRules(const AValue: TFWRules);
begin
  FRules.Assign(AValue);
end;

procedure TExtRadioGroup.p_setValues(const AValues: TStrings);
begin
  FValues.Assign(AValues);
end;

procedure TExtRadioGroup.Notification(AComponent: TComponent;
  Operation: TOperation);
var li_i : Integer;
begin // Erasing from external
  inherited Notification(AComponent, Operation);
  if ( Operation <> opRemove )
  or ( csDestroying in ComponentState ) Then
    Exit;
 for li_i := 0 to FRules.Count-1 do
  FRules [ li_i ].Notification(AComponent,Operation);
end;

constructor TExtRadioGroup.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  FRules:=ffwr_CreateRules;
  FValues := TStringList.Create;
end;

destructor TExtRadioGroup.Destroy;
begin
  inherited Destroy;
  FValues.Destroy;
  FRules.Destroy; // Collection
end;

function TExtRadioGroup.ffwr_CreateRules: TFWRules;
begin
  Result:=TFWRules.Create(Self,TFWRule);
end;

procedure TExtRadioGroup.CheckItemIndexChanged;
var li_i : Integer;
begin
  inherited;
  for li_i := 0 to FRules.Count - 1  do
    FRules [ li_i ].p_SetRuleToControl ( ItemIndex );
end;

{------------------------------------------------------------------------------
  Method: TExtCheckGroup.p_setRules
    Params:  avalue - Rules' collection
    Returns: Nothing

  Assign items from a stringlist.
 ------------------------------------------------------------------------------}
procedure TExtCheckGroup.p_setRules(const AValue: TFWRules);
begin
  FRules.Assign(AValue);
end;

procedure TExtCheckGroup.Notification(AComponent: TComponent;
  Operation: TOperation);
var li_i : Integer;
begin
  inherited Notification(AComponent, Operation);
  if ( Operation <> opRemove )
  or ( csDestroying in ComponentState ) Then
    Exit;
 for li_i := 0 to FRules.Count-1 do
  FRules [ li_i ].Notification(AComponent,Operation);
end;

constructor TExtCheckGroup.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  FRules:=ffwr_CreateRules;
end;

destructor TExtCheckGroup.Destroy;
begin
  inherited Destroy;
  FRules.Destroy;
end;

function TExtCheckGroup.ffwr_CreateRules: TFWRules;
begin
  Result:=TFWRules.Create(Self,TFWRule);
end;

procedure TExtCheckGroup.p_ItemClick (Sender: TObject; Index: integer);
var li_i : Integer;
begin
  inherited;
  for li_i := 0 to FRules.Count - 1  do
    FRules [ li_i ].p_SetRuleToControl ( Index );
end;

procedure TExtCheckGroup.Loaded;
begin
  inherited Loaded;
  FOldItemClick:=OnItemClick;
  OnItemClick:=p_ItemClick;
end;

{------------------------------------------------------------------------------
  Method: TExtDBRadioGroup.p_setRules
  Params:  avalue - Rules' collection
  Returns: Nothing

  Assign items from a stringlist.
 ------------------------------------------------------------------------------}

procedure TExtDBRadioGroup.p_setRules(const AValue: TFWRules);
begin
  FRules.Assign(AValue);
end;

procedure TExtDBRadioGroup.Notification(AComponent: TComponent;
  Operation: TOperation);
var li_i : Integer;
begin
  inherited Notification(AComponent, Operation);
  if ( Operation <> opRemove )
  or ( csDestroying in ComponentState ) Then
    Exit;
 for li_i := 0 to FRules.Count-1 do
  FRules [ li_i ].Notification(AComponent,Operation);
end;

constructor TExtDBRadioGroup.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  FRules:=ffwr_CreateRules;
end;

destructor TExtDBRadioGroup.Destroy;
begin
  inherited Destroy;
  FRules.Destroy;
end;

function TExtDBRadioGroup.ffwr_CreateRules: TFWRules;
begin
  Result:=TFWRules.Create(Self,TFWRule);
end;

procedure TExtDBRadioGroup.CheckItemIndexChanged;
var li_i : Integer;
begin
  inherited;
  for li_i := 0 to FRules.Count - 1  do
    FRules [ li_i ].p_SetRuleToControl ( ItemIndex );
end;

{$IFDEF VERSIONS}
initialization
  p_ConcatVersion ( gVer_extradios );
{$ENDIF}
end.
